@@ -38,10 +38,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkbindings.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "gdict-strategy-chooser.h"
 #include "gdict-utils.h"
+#include "gdict-debug.h"
 #include "gdict-private.h"
 #include "gdict-enum-types.h"
 #include "gdict-marshal.h"
@@ -84,6 +85,7 @@ enum
   STRAT_COLUMN_TYPE,
   STRAT_COLUMN_NAME,
   STRAT_COLUMN_DESCRIPTION,
+  STRAT_COLUMN_CURRENT,
 
   STRAT_N_COLUMNS
 };
@@ -124,7 +126,7 @@ set_gdict_context (GdictStrategyChooser *chooser,
     {
       if (priv->start_id)
         {
-          _gdict_debug ("Removing old context handlers\n");
+          GDICT_NOTE (CHOOSER, "Removing old context handlers");
           
           g_signal_handler_disconnect (priv->context, priv->start_id);
           g_signal_handler_disconnect (priv->context, priv->match_id);
@@ -142,9 +144,12 @@ set_gdict_context (GdictStrategyChooser *chooser,
           priv->error_id = 0;
         }
 
-      _gdict_debug ("Removing old context\n");
+      GDICT_NOTE (CHOOSER, "Removing old context");
       
       g_object_unref (G_OBJECT (priv->context));
+      
+      priv->context = NULL;
+      priv->results = -1;
     }
 
   if (!context)
@@ -157,10 +162,33 @@ set_gdict_context (GdictStrategyChooser *chooser,
       return;
     }
 
-  _gdict_debug ("Setting new context\n");
+  GDICT_NOTE (CHOOSER, "Setting new context");
     
-  priv->context = context;
-  g_object_ref (G_OBJECT (priv->context));
+  priv->context = g_object_ref (context);
+  priv->results = 0;
+}
+
+static void
+gdict_strategy_chooser_dispose (GObject *gobject)
+{
+  GdictStrategyChooser *chooser = GDICT_STRATEGY_CHOOSER (gobject);
+  GdictStrategyChooserPrivate *priv = chooser->priv;
+
+  set_gdict_context (chooser, NULL);
+
+  if (priv->busy_cursor)
+    {
+      gdk_cursor_unref (priv->busy_cursor);
+      priv->busy_cursor = NULL;
+    }
+
+  if (priv->store)
+    {
+      g_object_unref (priv->store);
+      priv->store = NULL;
+    }
+
+  G_OBJECT_CLASS (gdict_strategy_chooser_parent_class)->dispose (gobject);
 }
 
 static void
@@ -169,13 +197,7 @@ gdict_strategy_chooser_finalize (GObject *gobject)
   GdictStrategyChooser *chooser = GDICT_STRATEGY_CHOOSER (gobject);
   GdictStrategyChooserPrivate *priv = chooser->priv;
 
-  if (priv->context)
-    set_gdict_context (chooser, NULL);
-
-  if (priv->busy_cursor)
-    gdk_cursor_unref (priv->busy_cursor);
-
-  g_object_unref (priv->store);
+  g_free (priv->current_strat);
 
   G_OBJECT_CLASS (gdict_strategy_chooser_parent_class)->finalize (gobject);
 }
@@ -317,6 +339,7 @@ gdict_strategy_chooser_constructor (GType                  type,
   column = gtk_tree_view_column_new_with_attributes ("strategies",
 		  				     renderer,
 						     "text", STRAT_COLUMN_DESCRIPTION,
+                                                     "weight", STRAT_COLUMN_CURRENT,
 						     NULL);
   priv->treeview = gtk_tree_view_new ();
   gtk_widget_set_composite_name (priv->treeview, "gdict-strategy-chooser-treeview");
@@ -369,6 +392,7 @@ gdict_strategy_chooser_class_init (GdictStrategyChooserClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   
   gobject_class->finalize = gdict_strategy_chooser_finalize;
+  gobject_class->dispose = gdict_strategy_chooser_dispose;
   gobject_class->set_property = gdict_strategy_chooser_set_property;
   gobject_class->get_property = gdict_strategy_chooser_get_property;
   gobject_class->constructor = gdict_strategy_chooser_constructor;
@@ -421,9 +445,10 @@ gdict_strategy_chooser_init (GdictStrategyChooser *chooser)
   priv->context = NULL;
 
   priv->store = gtk_list_store_new (STRAT_N_COLUMNS,
-		                    G_TYPE_INT,    /* StratType */
-		                    G_TYPE_STRING, /* db_name */
-				    G_TYPE_STRING  /* db_desc */);
+		                    G_TYPE_INT,    /* strat_type */
+		                    G_TYPE_STRING, /* strat_name */
+				    G_TYPE_STRING, /* strat_desc */
+                                    G_TYPE_INT     /* strat_current */);
 
   priv->start_id = 0;
   priv->end_id = 0;
@@ -523,11 +548,41 @@ gdict_strategy_chooser_set_context (GdictStrategyChooser *chooser,
  */
 gchar **
 gdict_strategy_chooser_get_strategies (GdictStrategyChooser  *chooser,
-				      gsize                  length)
+                                       gsize                 *length)
 {
+  GdictStrategyChooserPrivate *priv;
+  GtkTreeIter iter;
+  gchar **retval;
+  gsize i;
+
   g_return_val_if_fail (GDICT_IS_STRATEGY_CHOOSER (chooser), NULL);
 
-  return NULL;
+  priv = chooser->priv;
+
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
+    return NULL;
+
+  i = 0;
+  retval = g_new (gchar*, priv->results);
+
+  do
+    {
+      gchar *strat_name;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                          STRAT_COLUMN_NAME, &strat_name,
+                          -1);
+
+      retval[i++] = strat_name;
+    }
+  while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter));
+
+  retval[i] = NULL;
+
+  if (length)
+    *length = i;
+
+  return retval;
 }
 
 /**
@@ -546,10 +601,40 @@ gboolean
 gdict_strategy_chooser_has_strategy (GdictStrategyChooser *chooser,
 				     const gchar          *strategy)
 {
+  GdictStrategyChooserPrivate *priv;
+  GtkTreeIter iter;
+  gboolean retval;
+
   g_return_val_if_fail (GDICT_IS_STRATEGY_CHOOSER (chooser), FALSE);
   g_return_val_if_fail (strategy != NULL, FALSE);
 
-  return FALSE;
+  priv = chooser->priv;
+
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
+    return FALSE;
+
+  retval = FALSE;
+
+  do
+    {
+      gchar *strat_name;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                          STRAT_COLUMN_NAME, &strat_name,
+                          -1);
+      
+      if (strcmp (strat_name, strategy) == 0)
+        {
+          retval = TRUE;
+          g_free (strat_name);
+          break;
+        }
+
+      g_free (strat_name);
+    }
+  while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter));
+
+  return retval;
 }
 
 /**
@@ -606,23 +691,29 @@ strategy_found_cb (GdictContext  *context,
 {
   GdictStrategyChooser *chooser = GDICT_STRATEGY_CHOOSER (user_data);
   GdictStrategyChooserPrivate *priv = chooser->priv;
+  const gchar *name, *description;
   GtkTreeIter iter;
+  gint weight = PANGO_WEIGHT_NORMAL;
 
-  _gdict_debug ("STRATEGY: `%s' (`%s')\n",
-		gdict_strategy_get_name (strategy),
-		gdict_strategy_get_description (strategy));
+  name = gdict_strategy_get_name (strategy);
+  description = gdict_strategy_get_description (strategy);
+
+  GDICT_NOTE (CHOOSER, "STRATEGY: `%s' (`%s')",
+              name,
+              description);
+
+  if (priv->current_strat && !strcmp (priv->current_strat, name))
+    weight = PANGO_WEIGHT_BOLD;
 
   gtk_list_store_append (priv->store, &iter);
   gtk_list_store_set (priv->store, &iter,
 		      STRAT_COLUMN_TYPE, STRATEGY_NAME,
-		      STRAT_COLUMN_NAME, gdict_strategy_get_name (strategy),
-		      STRAT_COLUMN_DESCRIPTION, gdict_strategy_get_description (strategy),
+		      STRAT_COLUMN_NAME, name,
+		      STRAT_COLUMN_DESCRIPTION, description,
+                      STRAT_COLUMN_CURRENT, weight,
 		      -1);
 
-  if (priv->results == -1)
-    priv->results = 1;
-  else
-    priv->results += 1;
+  priv->results += 1;
 }
 
 static void
@@ -636,6 +727,7 @@ error_cb (GdictContext *context,
     gdk_window_set_cursor (GTK_WIDGET (chooser)->window, NULL);
 
   chooser->priv->is_searching = FALSE;
+  chooser->priv->results = 0;
 }
 
 /**
@@ -665,12 +757,7 @@ gdict_strategy_chooser_refresh (GdictStrategyChooser *chooser)
     }
 
   if (priv->is_searching)
-    {
-      _gdict_show_error_dialog (NULL,
-				_("Another search is in progress"),
-				_("Please wait until the current search ends."));
-      return;
-    }
+    return;
 
   gdict_strategy_chooser_clear (chooser);
 
@@ -705,7 +792,9 @@ gdict_strategy_chooser_refresh (GdictStrategyChooser *chooser)
 			  STRAT_COLUMN_DESCRIPTION, NULL,
 			  -1);
 
-      _gdict_debug ("Error while searching: %s", db_error->message);
+      g_warning ("Error while retrieving strategies: %s",
+                 db_error->message);
+
       g_error_free (db_error);
     }
 }
@@ -730,7 +819,7 @@ gdict_strategy_chooser_clear (GdictStrategyChooser *chooser)
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview), NULL);
 
   gtk_list_store_clear (priv->store);
-  priv->results = -1;
+  priv->results = 0;
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview),
 		  	   GTK_TREE_MODEL (priv->store));
@@ -758,9 +847,6 @@ scan_for_strat_name (GtkTreeModel *model,
   if (!select_data)
     return TRUE;
 
-  if (select_data->found)
-    return TRUE;
-
   gtk_tree_model_get (model, iter, STRAT_COLUMN_NAME, &strat_name, -1);
   if (!strat_name)
     return FALSE;
@@ -774,8 +860,16 @@ scan_for_strat_name (GtkTreeModel *model,
 
       tree_view = GTK_TREE_VIEW (select_data->chooser->priv->treeview);
       if (select_data->do_activate)
-        gtk_tree_view_row_activated (tree_view, path,
-                                     gtk_tree_view_get_column (tree_view, 2));
+        {
+          GtkTreeViewColumn *column;
+
+          gtk_list_store_set (GTK_LIST_STORE (model), iter,
+                              STRAT_COLUMN_CURRENT, PANGO_WEIGHT_BOLD,
+                              -1);
+
+          column = gtk_tree_view_get_column (tree_view, 2);
+          gtk_tree_view_row_activated (tree_view, path, column);
+        }
 
       selection = gtk_tree_view_get_selection (tree_view);
       if (select_data->do_select)
@@ -783,10 +877,16 @@ scan_for_strat_name (GtkTreeModel *model,
       else
         gtk_tree_selection_unselect_path (selection, path);
     }
+  else
+    {
+      gtk_list_store_set (GTK_LIST_STORE (model), iter,
+                          STRAT_COLUMN_CURRENT, PANGO_WEIGHT_NORMAL,
+                          -1);
+    }
 
   g_free (strat_name);
 
-  return select_data->found;
+  return FALSE;
 }
 
 /**
@@ -805,7 +905,7 @@ gdict_strategy_chooser_select_strategy (GdictStrategyChooser *chooser,
                                         const gchar          *strat_name)
 {
   GdictStrategyChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_STRATEGY_CHOOSER (chooser), FALSE);
@@ -813,21 +913,19 @@ gdict_strategy_chooser_select_strategy (GdictStrategyChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->strat_name = g_strdup (strat_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = TRUE;
-  data->do_activate = FALSE;
+  data.strat_name = g_strdup (strat_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = TRUE;
+  data.do_activate = FALSE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_strat_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->strat_name);
-  g_slice_free (SelectData, data);
+  g_free (data.strat_name);
 
   return retval;
 }
@@ -849,7 +947,7 @@ gdict_strategy_chooser_unselect_strategy (GdictStrategyChooser *chooser,
                                           const gchar          *strat_name)
 {
   GdictStrategyChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_STRATEGY_CHOOSER (chooser), FALSE);
@@ -857,21 +955,19 @@ gdict_strategy_chooser_unselect_strategy (GdictStrategyChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->strat_name = g_strdup (strat_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = FALSE;
-  data->do_activate = FALSE;
+  data.strat_name = g_strdup (strat_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = FALSE;
+  data.do_activate = FALSE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_strat_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->strat_name);
-  g_slice_free (SelectData, data);
+  g_free (data.strat_name);
 
   return retval;
 }
@@ -892,7 +988,7 @@ gdict_strategy_chooser_set_current_strategy (GdictStrategyChooser *chooser,
                                              const gchar          *strat_name)
 {
   GdictStrategyChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_STRATEGY_CHOOSER (chooser), FALSE);
@@ -900,21 +996,28 @@ gdict_strategy_chooser_set_current_strategy (GdictStrategyChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->strat_name = g_strdup (strat_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = TRUE;
-  data->do_activate = TRUE;
+  if (priv->current_strat && !strcmp (priv->current_strat, strat_name))
+    return TRUE;
+
+  data.strat_name = g_strdup (strat_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = TRUE;
+  data.do_activate = TRUE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_strat_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->strat_name);
-  g_slice_free (SelectData, data);
+  if (data.found)
+    {
+      g_free (priv->current_strat);
+      priv->current_strat = data.strat_name;
+    }
+  else
+    g_free (data.strat_name);
 
   return retval;
 }
