@@ -27,7 +27,6 @@
 #include <config.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
-#include <glade/glade.h>
 #include <gdk/gdkx.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -87,6 +86,7 @@ typedef struct
   char *retval;
   int iteration;
   TestType type;
+  GdkWindow *window;
 } AsyncExistenceJob;
 
 static GdkPixbuf *screenshot = NULL;
@@ -676,6 +676,16 @@ screenshot_dialog_response_cb (GtkDialog *d,
         }
       g_free (uri);
     }
+  else if (response_id == SCREENSHOT_RESPONSE_COPY)
+    {
+      GtkClipboard *clipboard;
+      GdkPixbuf    *screenshot;
+
+      clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (GTK_WIDGET (d)),
+                                                 GDK_SELECTION_CLIPBOARD);
+      screenshot = screenshot_dialog_get_screenshot (dialog);
+      gtk_clipboard_set_image (clipboard, screenshot);
+    }
   else /* dialog was canceled */
     {
       gtk_widget_destroy (GTK_WIDGET (d));
@@ -700,36 +710,9 @@ run_dialog (ScreenshotDialog *dialog)
 }
 
 static void
-finish_prepare_screenshot (char *initial_uri)
+finish_prepare_screenshot (char *initial_uri, GdkWindow *window)
 {  
   ScreenshotDialog *dialog;
-  GdkWindow *window;
-
-  if (!screenshot_grab_lock ())
-    exit (0);
-
-  if (take_window_shot)
-    {
-      window = screenshot_find_current_window ();
-      if (!window)
-	{
-	  take_window_shot = FALSE;
-	  window = gdk_get_default_root_window ();
-	}
-      else
-	{
-	  gchar *tmp;
-
-	  window_title = screenshot_get_window_title (window);
-	  tmp = screenshot_sanitize_filename (window_title);
-	  g_free (window_title);
-	  window_title = tmp;
-	}
-    }
-  else
-    {
-      window = gdk_get_default_root_window ();
-    }
 
   screenshot = screenshot_get_pixbuf (window, include_pointer, include_border);
 
@@ -748,6 +731,7 @@ finish_prepare_screenshot (char *initial_uri)
       }
   }
 
+  /* release now the lock, it was acquired when we were finding the window */
   screenshot_release_lock ();
 
   if (screenshot == NULL)
@@ -770,13 +754,15 @@ static gboolean
 check_file_done (gpointer user_data)
 {
   char *retval;
+  GdkWindow *window;
   AsyncExistenceJob *job = user_data;
 
+  window = job->window;
   retval = job->retval;
   g_free (job->base_uris[1]);
   g_slice_free (AsyncExistenceJob, job);
   
-  finish_prepare_screenshot (retval);
+  finish_prepare_screenshot (retval, window);
   
   return FALSE;
 }
@@ -841,7 +827,7 @@ retry:
   error = NULL;
   uri = build_uri (job);
   file = g_file_new_for_uri (uri);
-  
+
   info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_TYPE,
 			    G_FILE_QUERY_INFO_NONE, cancellable, &error);
   if (info != NULL)
@@ -849,6 +835,8 @@ retry:
       /* file already exists, iterate again */
       g_object_unref (info);
       g_object_unref (file);
+      g_error_free (error);
+
       (job->iteration)++;
 
       goto retry;
@@ -889,7 +877,7 @@ retry:
 out:
   g_error_free (error);
   g_object_unref (file);
-    
+
   g_io_scheduler_job_send_to_mainloop_async (io_job,
                                              check_file_done,
                                              job,
@@ -897,6 +885,40 @@ out:
   return FALSE;
 }
 
+static GdkWindow *
+find_current_window (char **window_title)
+{
+  GdkWindow *window;
+
+  if (!screenshot_grab_lock ())
+    exit (0);
+
+  if (take_window_shot)
+    {
+      window = screenshot_find_current_window ();
+      if (!window)
+	{
+	  take_window_shot = FALSE;
+	  window = gdk_get_default_root_window ();
+	}
+      else
+	{
+	  gchar *tmp, *sanitized;
+
+	  tmp = screenshot_get_window_title (window);
+	  sanitized = screenshot_sanitize_filename (tmp);
+	  g_free (tmp);
+	  *window_title = sanitized;
+	}
+    }
+  else
+    {
+      window = gdk_get_default_root_window ();
+    }
+
+  return window;
+}
+
 static void
 prepare_screenshot (void)
 {
@@ -909,6 +931,7 @@ prepare_screenshot (void)
   job->base_uris[2] = (char *) g_get_tmp_dir ();
   job->iteration = 0;
   job->type = TEST_LAST_DIR;
+  job->window = find_current_window (&window_title);
 
   g_io_scheduler_push_job (try_check_file,
                            job,
@@ -1119,7 +1142,6 @@ main (int argc, char *argv[])
 				GNOME_PARAM_GOPTION_CONTEXT, context,
 				GNOME_PROGRAM_STANDARD_PROPERTIES,
 				NULL);
-  glade_gnome_init();
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
